# Copyright 2022 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Encapsulation of an HMC vault file in YAML format.

HMC vault files conform to the format of Ansible vault files in YAML
format and define specific variables for HMC authentication.
"""


from collections import OrderedDict
import errno
import yaml
import yamlloader
import jsonschema

__all__ = ['HMCVaultFileError', 'HMCVaultFile']

# Structure of an HMC vault file in YAML format:
#
#   hmc_auth:
#     <hmc_name>:  # DNS hostname, IP address, or nickname of HMC
#       userid: <userid>
#       password: <password>
#       verify: <verify>
#       ca_certs: <ca_certs>
#   <var_name>: <var_value>

# Regexp for valid Ansible variable names, HMC nicknames, group names
ANSIBLE_NAME_PATTERN = "^[a-zA-Z_][a-zA-Z_0-9]*$"

# Valid Ansible variable types (using JSON schema type names)
ANSIBLE_VARTYPES = [
    "object", "array", "string", "integer", "number", "boolean", "null"
]

# Regexp for valid Ansible hosts (DNS name, IP address, or HMC nickname)
# Note: Host name ranges like in Ansible are not supported.
ANSIBLE_HOST_PATTERN = r"^[a-zA-Z_0-9:\.\-]+$"

# JSON schema for content of an HMC vault file in YAML format
HMC_VAULT_FILE_SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema#",
    "title": "JSON schema for an HMC vault file in YAML format",
    "description": "HMC auth data and additional variables",
    "type": "object",
    "additionalProperties": False,
    "required": [
        "hmc_auth",
    ],
    "properties": {
        "hmc_auth": {
            "description": "Auth data for the HMCs covered by this vault file",
            "type": ["object", "null"],
            "additionalProperties": False,
            "patternProperties": {
                # HMC nickname:
                ANSIBLE_HOST_PATTERN: {
                    "description": "Auth data for a single HMC",
                    "type": "object",
                    "additionalProperties": False,
                    "required": [
                        "userid",
                        "password",
                    ],
                    "properties": {
                        "userid": {
                            "description":
                                "Userid (username) for authenticating with the "
                                "HMC. Required.",
                            "type": "string",
                        },
                        "password": {
                            "description":
                                "Password for authenticating with the HMC. "
                                "Required.",
                            "type": "string",
                        },
                        "verify": {
                            "description":
                                "Verify the HMC certificate as specified in "
                                "'ca_certs'. Optional, default: True.",
                            "type": "boolean",
                            "default": True,
                        },
                        "ca_certs": {
                            "description":
                                "Path name of certificate file or certificate "
                                "directory to be used for verifying the HMC "
                                "certificate, or None. If None, the path name "
                                "in the 'REQUESTS_CA_BUNDLE' environment "
                                "variable, or the path name in the "
                                "'CURL_CA_BUNDLE' environment variable, or the "
                                "certificates in the Mozilla CA Certificate "
                                "List provided by the 'certifi' Python package "
                                "are used. Optional, default: None.",
                            "type": "string",
                            "default": None,
                        },
                    },
                },
            },
        },
    },
    "patternProperties": {
        # Variable name:
        ANSIBLE_NAME_PATTERN: {
            "description": "Variable value",
            "type": ANSIBLE_VARTYPES,
        },
    },
}


class HMCVaultFileError(Exception):
    """
    An error in the :ref:`HMC vault file`.
    """
    pass


class HMCVaultFile:
    """
    Encapsulation of an :ref:`HMC vault file` in YAML format.
    """

    def __init__(self, filepath):
        self._filepath = filepath
        self._data = OrderedDict()  # file content
        self._load_file()

    def _load_file(self):
        """
        Load and validate the :ref:`HMC vault file` in YAML format.
        """
        try:
            # pylint: disable=unspecified-encoding
            with open(self._filepath) as fp:
                try:
                    data = yaml.load(
                        fp, Loader=yamlloader.ordereddict.SafeLoader)
                except (yaml.parser.ParserError,
                        yaml.scanner.ScannerError) as exc:
                    new_exc = HMCVaultFileError(
                        "Invalid YAML syntax in HMC vault file "
                        f"{self._filepath!r}: {exc.__class__.__name__} {exc}")
                    new_exc.__cause__ = None
                    raise new_exc  # HMCVaultFileError
        except OSError as exc:
            if exc.errno == errno.ENOENT:
                new_exc = HMCVaultFileError(
                    f"The HMC vault file {self._filepath!r} was not found")
                new_exc.__cause__ = None
                raise new_exc  # HMCVaultFileError
            raise

        try:
            jsonschema.validate(data, HMC_VAULT_FILE_SCHEMA)
        except jsonschema.exceptions.ValidationError as exc:
            elem = '.'.join(str(e) for e in exc.absolute_path)
            schemaitem = '.'.join(str(e) for e in exc.absolute_schema_path)
            new_exc = HMCVaultFileError(
                f"Invalid data format in HMC vault file {self._filepath}: "
                f"{exc.message}; "
                f"Offending element: {elem}; "
                f"Schema item: {schemaitem}; "
                f"Validator: {exc.validator}={exc.validator_value}")
            new_exc.__cause__ = None
            raise new_exc  # HMCVaultFileError

        self._data.update(data)

    def __repr__(self):
        return (
            "HMCVaultFile("
            f"filepath={self.filepath!r}, "
            f"data={self.data!r})")

    @property
    def filepath(self):
        """
        string: Path name of the :ref:`HMC vault file`.
        """
        return self._filepath

    @property
    def data(self):
        """
        :class:`~py:collections.OrderedDict`: Content of the
        :ref:`HMC vault file`, as nested
        :class:`~py:collections.OrderedDict` and
        :class:`~py:list` objects.
        """
        return self._data



# Copyright 2019,2021 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
End2end tests for CPCs.

These tests do not change any CPC properties.
"""


from datetime import timedelta, datetime, timezone
import pytest
from requests.packages import urllib3
import zhmcclient
# pylint: disable=line-too-long,unused-import
from zhmcclient.testutils import hmc_definition, hmc_session  # noqa: F401, E501
from zhmcclient.testutils import all_cpcs, classic_mode_cpcs, dpm_mode_cpcs  # noqa: F401, E501
# pylint: enable=line-too-long,unused-import

from .utils import skip_warn, assert_res_prop, runtest_find_list, \
    runtest_get_properties, validate_list_features, \
    cleanup_and_import_example_certificate, has_api_feature

urllib3.disable_warnings()

# Properties in minimalistic Cpc objects (e.g. find_by_name())
CPC_MINIMAL_PROPS = ['object-uri', 'name']

# Properties in Cpc objects returned by list() without full properties
CPC_LIST_PROPS = [
    'object-uri', 'name', 'status']
CPC_LIST_PROPS_Z15 = [
    'object-uri', 'name', 'status', 'has-unacceptable-status', 'dpm-enabled',
    'se-version']

# Properties whose values can change between retrievals
CPC_VOLATILE_PROPS = [
    'last-energy-advice-time',
    'zcpc-ambient-temperature',
    'zcpc-dew-point',
    'zcpc-exhaust-temperature',
    'zcpc-heat-load',
    'zcpc-heat-load-forced-air',
    'zcpc-heat-load-water',
    'zcpc-humidity',
    'zcpc-maximum-inlet-air-temperature',
    'zcpc-maximum-inlet-liquid-temperature',
    'zcpc-minimum-inlet-air-temperature',
    'zcpc-power-consumption',
    'cpc-power-consumption',
    'ec-mcl-description'  # TODO: Remove once 'last-update' volat. issue fixed
]

# Machine types with same max partitions for all models:
# Keep in sync with zhmcclient/_cpc.py.
MAX_PARTS_BY_TYPE = {
    '2817': 60,  # z196
    '2818': 30,  # z114
    '2827': 60,  # zEC12
    '2828': 30,  # zBC12
    '2964': 85,  # z13 / Emperor
    '2965': 40,  # z13s / Rockhopper
    '3906': 85,  # z14 / Emperor II
    '3907': 40,  # z14-ZR1 / Rockhopper II
    '8561': 85,  # z15-T01 / LinuxOne III (-LT1)
    '3931': 85,  # z16-A01
}

# Machine types with different max partitions across their models:
# Keep in sync with zhmcclient/_cpc.py.
MAX_PARTS_BY_TYPE_MODEL = {
    ('8562', 'T02'): 40,  # z15-T02
    ('8562', 'LT2'): 40,  # LinuxOne III (-LT2)
    ('8562', 'GT2'): 85,  # z15-GT2
}


def test_cpc_find_list(hmc_session):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test find/list methods for CPCs (any mode).
    """
    client = zhmcclient.Client(hmc_session)
    hd = hmc_session.hmc_definition

    for cpc_name in hd.cpcs:
        print(f"Testing with CPC {cpc_name}")

        cpc_list_props = CPC_LIST_PROPS
        se_version = hd.cpcs[cpc_name].get('se_version', None)
        if se_version:
            se_version_info = tuple(map(int, se_version.split('.')))
            if se_version_info >= (2, 15, 0):
                cpc_list_props = CPC_LIST_PROPS_Z15

        runtest_find_list(
            hmc_session, client.cpcs, cpc_name, 'name', 'status',
            CPC_VOLATILE_PROPS, CPC_MINIMAL_PROPS, cpc_list_props)


def test_cpc_property(all_cpcs):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test property related methods
    """
    if not all_cpcs:
        pytest.skip("HMC definition does not include any CPCs")

    for cpc in all_cpcs:
        print(f"Testing with CPC {cpc.name}")

        # Select a property that is not returned by list()
        non_list_prop = 'description'

        runtest_get_properties(cpc.manager, non_list_prop)


def test_cpc_features(all_cpcs):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test certain "features" of a CPC (any mode):
    - dpm_enabled property
    - maximum_active_partitions property
    - feature_enabled(feature_name)
    - feature_info()
    - list_api_features()
    """
    if not all_cpcs:
        pytest.skip("HMC definition does not include any CPCs")

    for cpc in all_cpcs:
        print(f"Testing with CPC {cpc.name}")

        cpc.pull_full_properties()
        cpc_mach_type = cpc.properties.get('machine-type', None)
        cpc_mach_model = cpc.properties.get('machine-model', None)
        cpc_features = cpc.properties.get('available-features-list', None)

        exp_cpc_props = dict(cpc.properties)

        # The code to be tested: dpm_enabled property
        dpm_enabled = cpc.dpm_enabled

        exp_dpm_enabled = exp_cpc_props.get('dpm-enabled', False)
        assert_res_prop(dpm_enabled, exp_dpm_enabled, 'dpm-enabled', cpc)

        # The code to be tested: maximum_active_partitions property
        max_parts = cpc.maximum_active_partitions

        exp_max_parts = exp_cpc_props.get('maximum-partitions', None)
        if cpc_mach_type and cpc_mach_model:
            if exp_max_parts is None:
                # Determine from tables
                try:
                    exp_max_parts = MAX_PARTS_BY_TYPE[cpc_mach_type]
                except KeyError:
                    exp_max_parts = MAX_PARTS_BY_TYPE_MODEL[
                        (cpc_mach_type, cpc_mach_model)]
        if exp_max_parts is not None:
            assert_res_prop(max_parts, exp_max_parts, 'maximum-partitions', cpc)

        # Test: feature_enabled(feature_name)
        feature_name = 'storage-management'
        if cpc_features is None:
            # The machine does not yet support features
            with pytest.raises(ValueError):
                # The code to be tested: feature_enabled(feature_name)
                cpc.feature_enabled(feature_name)
        else:
            features = [f for f in cpc_features if f['name'] == feature_name]
            if not features:
                # The machine generally supports features, but not this feature
                with pytest.raises(ValueError):
                    # The code to be tested: feature_enabled(feature_name)
                    cpc.feature_enabled(feature_name)
            else:
                # The machine supports this feature
                # The code to be tested: feature_enabled(feature_name)
                sm_enabled = cpc.feature_enabled(feature_name)
                exp_sm_enabled = features[0]['state']
                assert_res_prop(sm_enabled, exp_sm_enabled,
                                'available-features-list', cpc)

        # Test: feature_info()
        if cpc_features is None:
            # The machine does not yet support features
            with pytest.raises(ValueError):
                # The code to be tested: feature_info()
                cpc.feature_info()
        else:
            # The machine supports features
            # The code to be tested: feature_info()
            features = cpc.feature_info()
            # Note: It is possible that the feature list exists but is empty
            #       (e.g when a z14 HMC manages a z13)
            for i, feature in enumerate(features):
                assert 'name' in feature, (
                    f"Feature #{i} does not have the 'name' attribute "
                    f"in Cpc object for CPC {cpc.name}")
                assert 'description' in feature, (
                    f"Feature #{i} does not have the 'description' attribute "
                    f"in Cpc object for CPC {cpc.name}")
                assert 'state' in feature, (
                    f"Feature #{i} does not have the 'state' attribute "
                    f"in Cpc object for CPC {cpc.name}")

        # Test: list_api_features()
        client = zhmcclient.Client(cpc.manager.session)

        validate_list_features(client.query_api_version(),
                               cpc.list_api_features(),
                               cpc.list_api_features('cpc.*'))


def test_cpc_export_profiles(classic_mode_cpcs):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test for export_profiles(profile_area, wait_for_completion=True,
                             operation_timeout=None)

    Only for CPCs in classic mode, skipped in DPM mode.
    """
    if not classic_mode_cpcs:
        pytest.skip("HMC definition does not include any CPCs in classic mode")

    for cpc in classic_mode_cpcs:
        assert not cpc.dpm_enabled

        session = cpc.manager.session
        hd = session.hmc_definition

        print(f"Testing with CPC {cpc.name}")

        try:

            # The code to be tested
            _ = cpc.export_profiles(profile_area=1)

        except zhmcclient.HTTPError as exc:
            if exc.http_status == 403 and exc.reason == 1:
                skip_warn(
                    f"HMC userid {hd.userid!r} is not authorized for task "
                    f"'Export/Import Profile Data (API only)' on HMC {hd.host}")
            else:
                raise

        # TODO: Complete this test


def test_cpc_export_dpm_config(dpm_mode_cpcs):  # noqa: F811
    # pylint: disable=redefined-outer-name
    """
    Test for export_dpm_configuration()

    Only for CPCs in DPM mode, skipped in classic mode.

    Note: export_dpm_configuration() is time-consuming and is invoked twice, so
    depending on the configuration of the CPCs, this test might easily run
    for 5 to 20 minutes!
    """
    if not dpm_mode_cpcs:
        pytest.skip("HMC definition does not include any CPCs in DPM mode")

    for cpc in dpm_mode_cpcs:
        assert cpc.dpm_enabled

        print(f"Testing on CPC {cpc.name}")

        cert, cert_props = (None, None)
        if has_api_feature('secure-boot-with-certificates', cpc):
            cert, cert_props = cleanup_and_import_example_certificate(cpc)

        config = cpc.export_dpm_configuration()
        full_config = \
            cpc.export_dpm_configuration(include_unused_adapters=True)

        assert len(config['adapters']) <= len(full_config['adapters'])

        if has_api_feature('secure-boot-with-certificates', cpc):
            assert len(config['certificates']) >= 1
            for exported_cert in config['certificates']:
                if exported_cert['name'] == cert_props['name']:
                    assert exported_cert['certificate'] == \
                           cert_props['certificate']
            cert.delete()


TESTCASES_CPC_GET_SUSTAINABILITY_DATA = [
    # Test cases for test_cpc_get_sustainability_data(), each as a tuple with
    # these items:
    # * tc: Testcase short description
    # * input_kwargs: kwargs to be used as input parameters for
    #   Cpc.get_sustainability_data()
    # * exp_oldest: expected delta time from now to oldest data point,
    #   as timedelta
    # * exp_delta: expected delta time between data points, as timedelta
    (
        "Default values (range=last-week, resolution=one-hour)",
        {},
        timedelta(days=7),
        timedelta(hours=1),
    ),
    (
        "range=last-day, resolution=one-hour",
        {
            "range": "last-day",
            "resolution": "one-hour",
        },
        timedelta(hours=24),
        timedelta(hours=1),
    ),
    (
        "range=last-day, resolution=fifteen-minutes",
        {
            "range": "last-day",
            "resolution": "fifteen-minutes",
        },
        timedelta(hours=24),
        timedelta(minutes=15),
    ),
]

CPC_METRICS = {
    # Metrics returned in "Get CPC Historical Sustainability Data" HMC
    # operation.
    # metric name: data type
    "total-wattage": int,
    "partition-wattage": int,
    "infrastructure-wattage": int,
    "unassigned-wattage": int,
    "heat-load": int,
    "heat-load-forced-air": int,
    "processor-utilization": int,
    "ambient-temperature": float,
    "exhaust-heat-temperature": float,
    "dew-point": float,
    "ambient-humidity": int,
}


@pytest.mark.parametrize(
    "tc, input_kwargs, exp_oldest, exp_delta",
    TESTCASES_CPC_GET_SUSTAINABILITY_DATA)
def test_cpc_get_sustainability_data(
        tc, input_kwargs, exp_oldest, exp_delta, all_cpcs):  # noqa: F811
    # pylint: disable=redefined-outer-name,unused-argument
    """
    Test for Cpc.get_sustainability_data(...)
    """
    for cpc in all_cpcs:

        session = cpc.manager.session
        hd = session.hmc_definition

        print(f"Testing with CPC {cpc.name}")

        try:

            # The code to be tested
            data = cpc.get_sustainability_data(**input_kwargs)

        except zhmcclient.HTTPError as exc:
            if exc.http_status == 403 and exc.reason == 1:
                skip_warn(
                    f"HMC userid {hd.userid!r} is not authorized for task "
                    f"'Environmental Dashboard' on HMC {hd.host}")
            elif exc.http_status == 404 and exc.reason == 1:
                skip_warn(
                    f"CPC {cpc.name} on HMC {hd.host} does not support "
                    f"feature: {exc}")
            else:
                raise

        now_dt = datetime.now(timezone.utc)
        exp_oldest_dt = now_dt - exp_oldest

        act_metric_names = set(data.keys())
        exp_metric_names = set(CPC_METRICS.keys())
        assert act_metric_names == exp_metric_names

        for metric_name, metric_array in data.items():
            metric_type = CPC_METRICS[metric_name]

            first_item = True
            previous_dt = None
            for dp_item in metric_array:
                # We assume the order is oldest to newest

                assert 'data' in dp_item
                assert 'timestamp' in dp_item
                assert len(dp_item) == 2

                dp_data = dp_item['data']
                dp_timestamp_dt = dp_item['timestamp']

                assert isinstance(dp_data, metric_type), \
                    f"Invalid data type for metric {metric_name!r}"

                if first_item:
                    first_item = False

                    # Verify that the oldest timestamp is within a certain
                    # delta from the range start.
                    # There are cases where that is not satisfied, so we only
                    # issue only a warning (as opposed to failing).
                    delta_sec = abs((dp_timestamp_dt - exp_oldest_dt).seconds)
                    if delta_sec > 15 * 60:
                        print("Warning: Oldest data point of metric "
                              f"{metric_name!r} is not within 15 minutes of "
                              "range start: Oldest data point: "
                              f"{dp_timestamp_dt}, Range start: "
                              f"{exp_oldest_dt}, Delta: {delta_sec} sec")
                else:

                    # For second oldest timestamp on, verify that the delta
                    # to the previous data point is the requested resolution.
                    # There are cases where that is not satisfied, so we only
                    # issue only a warning (as opposed to failing).
                    tolerance_pct = 10
                    delta_td = abs(dp_timestamp_dt - previous_dt)
                    if abs(delta_td.seconds - exp_delta.seconds) > \
                            tolerance_pct / 100 * exp_delta.seconds:
                        print("Warning: Timestamp of a data point of metric "
                              f"{metric_name!r} is not within expected delta "
                              "of its previous data point. Actual delta: "
                              f"{delta_td}, Expected delta: {exp_delta} "
                              f"(+/-{tolerance_pct}%)")

                previous_dt = dp_timestamp_dt


# Read-only tests:
# TODO: Test for get_wwpns(partitions)
# TODO: Test for get_free_crypto_domains(crypto_adapters=None)
# TODO: Test for get_energy_management_properties()
# TODO: Test for validate_lun_path(host_wwpn, host_port, wwpn, lun)
# TODO: Test for export_dpm_configuration(include_unused_adapters)

# Modifying tests:
# TODO: Test for update_properties(properties)
# TODO: Test for start(wait_for_completion=True, operation_timeout=None)
# TODO: Test for stop(wait_for_completion=True, operation_timeout=None)
# TODO: Test for import_profiles(profile_area, wait_for_completion=True,
#                                operation_timeout=None)
# TODO: Test for set_power_save(power_saving, wait_for_completion=True,
#                               operation_timeout=None)
# TODO: Test for set_power_capping(power_capping_state, power_cap=None,
#                                  wait_for_completion=True,
#                                  operation_timeout=None)
# TODO: Test for add_temporary_capacity(record_id, software_model=None,
#                                       processor_info=None, test=False,
#                                       force=False)
# TODO: Test for remove_temporary_capacity(record_id, software_model=None,
#                                          processor_info=None)
# TODO: Test for set_auto_start_list(auto_start_list)
# TODO: Test for import_dpm_configuration(dpm_configuration)



# Copyright 2018,2021 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _storage_group module.
"""


import re
import copy
import pytest

from zhmcclient import Client, Cpc, StorageGroup, StorageGroupManager, \
    StorageVolumeManager, VirtualStorageResourceManager, HTTPError, \
    NotFound
from zhmcclient_mock import FakedSession
from tests.common.utils import assert_resources


# Object IDs and names of our faked storage groups:
CPC_OID = 'fake-cpc1-oid'
CPC_URI = f'/api/cpcs/{CPC_OID}'
SG1_OID = 'sg1-oid'
SG1_NAME = 'sg 1'
SG2_OID = 'sg2-oid'
SG2_NAME = 'sg 2'


class TestStorageGroup:
    """All tests for the StorageGroup and StorageGroupManager classes."""

    def setup_method(self):
        """
        Setup that is called by pytest before each test method.

        Set up a faked session, and add a faked CPC in DPM mode without any
        child resources.
        """
        # pylint: disable=attribute-defined-outside-init

        self.session = FakedSession('fake-host', 'fake-hmc', '2.14.1', '1.9')
        self.client = Client(self.session)
        self.faked_cpc = self.session.hmc.cpcs.add({
            'object-id': CPC_OID,
            # object-uri is set up automatically
            'parent': None,
            'class': 'cpc',
            'name': 'fake-cpc1-name',
            'description': 'CPC #1 (DPM mode, storage mgmt feature enabled)',
            'status': 'active',
            'dpm-enabled': True,
            'is-ensemble-member': False,
            'iml-mode': 'dpm',
            'available-features-list': [
                dict(name='dpm-storage-management', state=True),
            ],
        })
        assert self.faked_cpc.uri == CPC_URI
        self.cpc = self.client.cpcs.find(name='fake-cpc1-name')
        self.faked_console = self.session.hmc.consoles.add({
            # object-id is set up automatically
            # object-uri is set up automatically
            # parent will be automatically set
            # class will be automatically set
            'name': 'fake-console-name',
            'description': 'The HMC',
        })
        self.console = self.client.consoles.console

    def add_storage_group1(self):
        """Add storage group 1 (type fcp)."""

        faked_storage_group = self.faked_console.storage_groups.add({
            'object-id': SG1_OID,
            # object-uri will be automatically set
            # parent will be automatically set
            # class will be automatically set
            'cpc-uri': CPC_URI,
            'name': SG1_NAME,
            'description': 'Storage Group #1',
            'type': 'fcp',
            'shared': False,
            'fulfillment-state': 'complete',
            'connectivity': 4,
        })
        return faked_storage_group

    def add_storage_group2(self):
        """Add storage group 2 (type fc)."""

        faked_storage_group = self.faked_console.storage_groups.add({
            'object-id': SG2_OID,
            # object-uri will be automatically set
            # parent will be automatically set
            # class will be automatically set
            'cpc-uri': CPC_URI,
            'name': SG2_NAME,
            'description': 'Storage Group #2',
            'type': 'fc',
            'shared': False,
            'fulfillment-state': 'complete',
            'connectivity': 4,
        })
        return faked_storage_group

    def test_sm_initial_attrs(self):
        """Test initial attributes of StorageGroupManager."""

        storage_group_mgr = self.console.storage_groups

        assert isinstance(storage_group_mgr, StorageGroupManager)

        # Verify all public properties of the manager object
        assert storage_group_mgr.resource_class == StorageGroup
        assert storage_group_mgr.session == self.session
        assert storage_group_mgr.parent == self.console
        assert storage_group_mgr.console == self.console

    # TODO: Test for StorageGroupManager.__repr__()

    testcases_sm_list_full_properties = (
        "full_properties_kwargs, prop_names", [
            ({},
             ['object-uri', 'cpc-uri', 'name', 'fulfillment-state', 'type']),
            (dict(full_properties=False),
             ['object-uri', 'cpc-uri', 'name', 'fulfillment-state', 'type']),
            (dict(full_properties=True),
             None),
        ]
    )

    @pytest.mark.parametrize(
        *testcases_sm_list_full_properties
    )
    def test_sm_list_full_properties(
            self, full_properties_kwargs, prop_names):
        """Test StorageGroupManager.list() with full_properties."""

        # Add two faked storage groups
        faked_storage_group1 = self.add_storage_group1()
        faked_storage_group2 = self.add_storage_group2()

        exp_faked_storage_groups = [faked_storage_group1, faked_storage_group2]
        storage_group_mgr = self.console.storage_groups

        # Execute the code to be tested
        storage_groups = storage_group_mgr.list(**full_properties_kwargs)

        assert_resources(storage_groups, exp_faked_storage_groups, prop_names)

    testcases_sm_list_filter_args = (
        "filter_args, exp_names", [
            ({'object-id': SG1_OID},
             [SG1_NAME]),
            ({'object-id': SG2_OID},
             [SG2_NAME]),
            ({'object-id': [SG1_OID, SG2_OID]},
             [SG1_NAME, SG2_NAME]),
            ({'object-id': [SG1_OID, SG1_OID]},
             [SG1_NAME]),
            ({'object-id': SG1_OID + 'foo'},
             []),
            ({'object-id': [SG1_OID, SG2_OID + 'foo']},
             [SG1_NAME]),
            ({'object-id': [SG2_OID + 'foo', SG1_OID]},
             [SG1_NAME]),
            ({'name': SG1_NAME},
             [SG1_NAME]),
            ({'name': SG2_NAME},
             [SG2_NAME]),
            ({'name': [SG1_NAME, SG2_NAME]},
             [SG1_NAME, SG2_NAME]),
            ({'name': SG1_NAME + 'foo'},
             []),
            ({'name': [SG1_NAME, SG2_NAME + 'foo']},
             [SG1_NAME]),
            ({'name': [SG2_NAME + 'foo', SG1_NAME]},
             [SG1_NAME]),
            ({'name': [SG1_NAME, SG1_NAME]},
             [SG1_NAME]),
            ({'name': '.*sg 1'},
             [SG1_NAME]),
            ({'name': 'sg 1.*'},
             [SG1_NAME]),
            ({'name': 'sg .'},
             [SG1_NAME, SG2_NAME]),
            ({'name': '.g 1'},
             [SG1_NAME]),
            ({'name': '.+'},
             [SG1_NAME, SG2_NAME]),
            ({'name': 'sg 1.+'},
             []),
            ({'name': '.+sg 1'},
             []),
            ({'name': SG1_NAME,
              'object-id': SG1_OID},
             [SG1_NAME]),
            ({'name': SG1_NAME,
              'object-id': SG1_OID + 'foo'},
             []),
            ({'name': SG1_NAME + 'foo',
              'object-id': SG1_OID},
             []),
            ({'name': SG1_NAME + 'foo',
              'object-id': SG1_OID + 'foo'},
             []),
        ]
    )

    @pytest.mark.parametrize(
        *testcases_sm_list_filter_args
    )
    def test_sm_list_filter_args(
            self, filter_args, exp_names):
        """Test StorageGroupManager.list() with filter_args."""

        # Add two faked storage_groups
        self.add_storage_group1()
        self.add_storage_group2()

        storage_group_mgr = self.console.storage_groups

        # Execute the code to be tested
        storage_groups = storage_group_mgr.list(filter_args=filter_args)

        assert len(storage_groups) == len(exp_names)
        if exp_names:
            names = [p.properties['name'] for p in storage_groups]
            assert set(names) == set(exp_names)

    testcases_sm_create_no_volumes = (
        "input_props, exp_prop_names, exp_exc", [
            ({},
             None,
             HTTPError({'http-status': 400, 'reason': 5})),
            ({'description': 'fake description X'},
             None,
             HTTPError({'http-status': 400, 'reason': 5})),
            ({'name': 'fake-sg-x'},
             None,
             HTTPError({'http-status': 400, 'reason': 5})),
            ({'name': 'fake-sg-x',
              'cpc-uri': CPC_URI,
              'type': 'fcp'},
             ['object-uri', 'name', 'cpc-uri'],
             None),
        ]
    )

    @pytest.mark.parametrize(
        *testcases_sm_create_no_volumes
    )
    def test_sm_create(
            self, input_props, exp_prop_names, exp_exc):
        """Test StorageGroupManager.create()."""

        # TODO: Add logic and test cases for creating initial storage volumes

        storage_group_mgr = self.console.storage_groups

        if exp_exc is not None:

            with pytest.raises(exp_exc.__class__) as exc_info:

                # Execute the code to be tested
                storage_group = storage_group_mgr.create(
                    properties=input_props)

            exc = exc_info.value
            if isinstance(exp_exc, HTTPError):
                assert exc.http_status == exp_exc.http_status
                assert exc.reason == exp_exc.reason

        else:

            # Execute the code to be tested.
            # Note: the StorageGroup object returned by StorageGroup.create()
            # has the input properties plus 'object-uri'.
            storage_group = storage_group_mgr.create(properties=input_props)

            # Check the resource for consistency within itself
            assert isinstance(storage_group, StorageGroup)
            storage_group_name = storage_group.name
            exp_storage_group_name = storage_group.properties['name']
            assert storage_group_name == exp_storage_group_name
            storage_group_uri = storage_group.uri
            exp_storage_group_uri = storage_group.properties['object-uri']
            assert storage_group_uri == exp_storage_group_uri

            # Check the properties against the expected names and values
            for prop_name in exp_prop_names:
                assert prop_name in storage_group.properties
                if prop_name in input_props:
                    value = storage_group.properties[prop_name]
                    exp_value = input_props[prop_name]
                    assert value == exp_value

    def test_sm_resource_object(self):
        """
        Test StorageGroupManager.resource_object().

        This test exists for historical reasons, and by now is covered by the
        test for BaseManager.resource_object().
        """

        # Add a faked storage_group
        faked_storage_group = self.add_storage_group1()
        storage_group_oid = faked_storage_group.oid

        storage_group_mgr = self.console.storage_groups

        # Execute the code to be tested
        storage_group = storage_group_mgr.resource_object(storage_group_oid)

        storage_group_uri = "/api/storage-groups/" + storage_group_oid

        sv_mgr = storage_group.storage_volumes
        vsr_mgr = storage_group.virtual_storage_resources

        assert isinstance(storage_group, StorageGroup)
        assert isinstance(sv_mgr, StorageVolumeManager)
        assert isinstance(vsr_mgr, VirtualStorageResourceManager)

        sg_cpc = storage_group.cpc
        assert isinstance(sg_cpc, Cpc)
        assert sg_cpc.uri == storage_group.properties['cpc-uri']

        # Note: Properties inherited from BaseResource are tested there,
        # but we test them again:
        assert storage_group.properties['object-uri'] == storage_group_uri
        assert storage_group.properties['object-id'] == storage_group_oid
        assert storage_group.properties['class'] == 'storage-group'
        assert storage_group.properties['parent'] == self.console.uri

    def test_sg_repr(self):
        """Test StorageGroup.__repr__()."""

        # Add a faked storage_group
        faked_storage_group = self.add_storage_group1()

        storage_group_mgr = self.console.storage_groups
        storage_group = storage_group_mgr.find(name=faked_storage_group.name)

        # Execute the code to be tested
        repr_str = repr(storage_group)

        repr_str = repr_str.replace('\n', '\\n')
        # We check just the begin of the string:
        assert re.match(
            rf'^{storage_group.__class__.__name__}\s+at\s+'
            rf'0x{id(storage_group):08x}\s+\(\\n.*',
            repr_str)

    def test_sg_delete_non_associated(self):
        """Test StorageGroup.delete() of non-associated storage group."""

        # Add a faked storage group to be tested and another one
        faked_storage_group = self.add_storage_group1()
        self.add_storage_group2()

        storage_group_mgr = self.console.storage_groups

        storage_group = storage_group_mgr.find(name=faked_storage_group.name)

        # Execute the code to be tested.
        storage_group.delete()

        # Check that the storage group no longer exists
        with pytest.raises(NotFound):
            storage_group_mgr.find(name=faked_storage_group.name)

    def test_sg_delete_create_same(self):
        """Test StorageGroup.delete() followed by create() with same name."""

        # Add a faked storage_group to be tested and another one
        faked_storage_group = self.add_storage_group1()
        storage_group_name = faked_storage_group.name
        self.add_storage_group2()

        # Construct the input properties for a third storage_group
        sg3_props = copy.deepcopy(faked_storage_group.properties)
        sg3_props['description'] = 'Third storage_group'

        storage_group_mgr = self.console.storage_groups
        storage_group = storage_group_mgr.find(name=storage_group_name)

        # Execute the deletion code to be tested.
        storage_group.delete()

        # Check that the storage_group no longer exists
        with pytest.raises(NotFound):
            storage_group_mgr.find(name=storage_group_name)

        # Execute the creation code to be tested.
        storage_group_mgr.create(sg3_props)

        # Check that the storage_group exists again under that name
        storage_group3 = storage_group_mgr.find(name=storage_group_name)
        description = storage_group3.get_property('description')
        assert description == 'Third storage_group'

    testcases_sg_update_properties_sgs = (
        "storage_group_name", [
            SG1_NAME,
            SG2_NAME,
        ]
    )

    testcases_sg_update_properties_props = (
        "input_props", [
            {},
            {'description': 'New storage_group description'},
            {'shared': True},
            {'connectivity': 8},
        ]
    )

    @pytest.mark.parametrize(
        *testcases_sg_update_properties_sgs
    )
    @pytest.mark.parametrize(
        *testcases_sg_update_properties_props
    )
    def test_sg_update_properties(
            self, input_props, storage_group_name):
        """Test StorageGroup.update_properties()."""

        # Add faked storage_groups
        self.add_storage_group1()
        self.add_storage_group2()

        storage_group_mgr = self.console.storage_groups
        storage_group = storage_group_mgr.find(name=storage_group_name)

        storage_group.pull_full_properties()
        saved_properties = copy.deepcopy(storage_group.properties)

        # Execute the code to be tested
        storage_group.update_properties(properties=input_props)

        # Verify that the resource object already reflects the property
        # updates.
        for prop_name in saved_properties:
            if prop_name in input_props:
                exp_prop_value = input_props[prop_name]
            else:
                exp_prop_value = saved_properties[prop_name]
            assert prop_name in storage_group.properties
            prop_value = storage_group.properties[prop_name]
            assert prop_value == exp_prop_value

        # Refresh the resource object and verify that the resource object
        # still reflects the property updates.
        storage_group.pull_full_properties()
        for prop_name in saved_properties:
            if prop_name in input_props:
                exp_prop_value = input_props[prop_name]
            else:
                exp_prop_value = saved_properties[prop_name]
            assert prop_name in storage_group.properties
            prop_value = storage_group.properties[prop_name]
            assert prop_value == exp_prop_value

    def test_sg_update_name(self):
        """
        Test StorageGroup.update_properties() with 'name' property.
        """

        # Add a faked storage_group
        faked_storage_group = self.add_storage_group1()
        storage_group_name = faked_storage_group.name

        storage_group_mgr = self.console.storage_groups
        storage_group = storage_group_mgr.find(name=storage_group_name)

        new_storage_group_name = "new-" + storage_group_name

        # Execute the code to be tested
        storage_group.update_properties(
            properties={'name': new_storage_group_name})

        # Verify that the resource is no longer found by its old name, using
        # list() (this does not use the name-to-URI cache).
        storage_groups_list = storage_group_mgr.list(
            filter_args=dict(name=storage_group_name))
        assert len(storage_groups_list) == 0

        # Verify that the resource is no longer found by its old name, using
        # find() (this uses the name-to-URI cache).
        with pytest.raises(NotFound):
            storage_group_mgr.find(name=storage_group_name)

        # Verify that the resource object already reflects the update, even
        # though it has not been refreshed yet.
        assert storage_group.properties['name'] == new_storage_group_name

        # Refresh the resource object and verify that it still reflects the
        # update.
        storage_group.pull_full_properties()
        assert storage_group.properties['name'] == new_storage_group_name

        # Verify that the resource can be found by its new name, using find()
        new_storage_group_find = storage_group_mgr.find(
            name=new_storage_group_name)
        assert new_storage_group_find.properties['name'] == \
            new_storage_group_name

        # Verify that the resource can be found by its new name, using list()
        new_storage_groups_list = storage_group_mgr.list(
            filter_args=dict(name=new_storage_group_name))
        assert len(new_storage_groups_list) == 1
        new_storage_group_list = new_storage_groups_list[0]
        assert new_storage_group_list.properties['name'] == \
            new_storage_group_name

    @pytest.mark.parametrize(
        "initial_status, exp_exc", [
            ('stopped', None),
            ('terminated', HTTPError({'http-status': 409, 'reason': 1})),
            ('starting', HTTPError({'http-status': 409, 'reason': 1})),
            ('active', HTTPError({'http-status': 409, 'reason': 1})),
            ('stopping', HTTPError({'http-status': 409, 'reason': 1})),
            ('degraded', HTTPError({'http-status': 409, 'reason': 1})),
            ('reservation-error',
             HTTPError({'http-status': 409, 'reason': 1})),
            ('paused', HTTPError({'http-status': 409, 'reason': 1})),
        ]
    )
    @pytest.mark.skip("TODO: Adjust to invoke a SG method")
    def test_storagegroup_start(self, initial_status, exp_exc):
        """Test StorageGroup.start()."""

        # Add a faked storage_group
        faked_storage_group = self.add_storage_group1()

        # Set the initial status of the faked storage_group
        faked_storage_group.properties['status'] = initial_status

        storage_group_mgr = self.console.storage_groups
        storage_group = storage_group_mgr.find(name=faked_storage_group.name)

        if exp_exc is not None:

            with pytest.raises(exp_exc.__class__) as exc_info:

                # Execute the code to be tested
                storage_group.start()

            exc = exc_info.value
            if isinstance(exp_exc, HTTPError):
                assert exc.http_status == exp_exc.http_status
                assert exc.reason == exp_exc.reason

        else:

            # Execute the code to be tested.
            ret = storage_group.start()

            assert ret == {}

            storage_group.pull_full_properties()
            status = storage_group.get_property('status')
            assert status == 'active'

    # TODO: Test for StorageGroup.list_attached_partitions()

    # TODO: Test for StorageGroup.add_candidate_adapter_ports()

    # TODO: Test for StorageGroup.remove_candidate_adapter_ports()

    # TODO: Test for StorageGroup.list_candidate_adapter_ports()



#!/usr/bin/env python
# Copyright 2016,2022 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example that mounts an ISO image to a partition and starts the partition
on a CPC in DPM mode.
"""

import sys
import os
import uuid
import requests.packages.urllib3

import zhmcclient
from zhmcclient.testutils import hmc_definitions

requests.packages.urllib3.disable_warnings()

# Get HMC info from HMC inventory and vault files
hmc_def = hmc_definitions()[0]
nickname = hmc_def.nickname
host = hmc_def.host
userid = hmc_def.userid
password = hmc_def.password
verify_cert = hmc_def.verify_cert

# Customize: Set to the ISO image you want to mount:
# path name of image file on local system
image_file = 'try/SLE-12-Server-DVD-s390x-GM-DVD1.iso'
# path name of INS file within ISO image, or None
image_insfile = 'my_insfile'

print(__doc__)

print(f"Using HMC {nickname} at {host} with userid {userid} ...")

print("Creating a session with the HMC ...")
try:
    session = zhmcclient.Session(
        host, userid, password, verify_cert=verify_cert)
except zhmcclient.Error as exc:
    print(f"Error: Cannot establish session with HMC {host}: "
          f"{exc.__class__.__name__}: {exc}")
    sys.exit(1)

try:
    client = zhmcclient.Client(session)

    print("Finding a CPC in DPM mode ...")
    cpcs = client.cpcs.list(filter_args={'dpm-enabled': True})
    if not cpcs:
        print(f"Error: HMC at {host} does not manage any CPCs in DPM mode")
        sys.exit(1)
    cpc = cpcs[0]
    print(f"Using CPC {cpc.name}")

    part_name = f"zhmc_test_{uuid.uuid4()}"
    print(f"Creating partition {part_name} ...")
    try:
        part = cpc.partitions.create(
            properties={
                'name': part_name,
                'type': 'linux',
                'ifl-processors': 2,
                'initial-memory': 4096,
                'maximum-memory': 4096,
            })
    except zhmcclient.Error as exc:
        print(f"Error: Cannot create partition {part_name} on CPC {cpc.name}: "
              f"{exc.__class__.__name__}: {exc}")
        sys.exit(1)

    try:
        try:
            with open(image_file, 'rb') as image_fp:
                image_name = os.path.basename(image_file)
                image_size_mb = 1.0 * os.path.getsize(image_file) / 1024 / 1024
                print(f"Mounting ISO image to partition {part.name} ...")
                print(f"  Image file: {image_file} (size: {image_size_mb:.1f} MB)")
                print(f"  Image name: {image_name}")
                print(f"  Image INS file: {image_insfile}")
                try:
                    part.mount_iso_image(image_fp, image_name, image_insfile)
                except zhmcclient.Error as exc:
                    print(f"Error: Cannot mount ISO file {image_file}: "
                          f"{exc.__class__.__name__}: {exc}")
                    sys.exit(1)
        except OSError as exc:
            print(f"Error: Cannot open image file {image_file}: "
                  f"{exc.__class__.__name__}: {exc}")
            sys.exit(1)

        part.pull_full_properties()
        image_name = part.get_property('boot-iso-image-name')
        print("Partition property 'boot-iso-image-name' has been set to image "
              f"name: {image_name}")

        print("Setting 'iso-image' as a boot device ...")
        try:
            part.update_properties({'boot-device': 'iso-image'})
        except zhmcclient.Error as exc:
            print(f"Error: Cannot update properties of partition {part.name}: "
                  f"{exc.__class__.__name__}: {exc}")
            sys.exit(1)

        print(f"Starting partition {part.name} ...")
        try:
            part.start()
        except zhmcclient.Error as exc:
            print(f"Error: Cannot start partition {part.name}: "
                  f"{exc.__class__.__name__}: {exc}")
            sys.exit(1)

        part.pull_full_properties()
        status = part.get_property('status')
        print(f"Partition status: {status}")

    finally:
        if part.get_property('status') != 'stopped':
            print(f"Stopping partition {part.name} ...")
            try:
                part.stop(wait_for_completion=True)
            except zhmcclient.Error as exc:
                print("Error: Stop operation failed with "
                      f"{exc.__class__.__name__}: {exc}")
                sys.exit(1)

        print(f"Deleting partition {part.name} ...")
        try:
            part.delete()
        except zhmcclient.Error as exc:
            print(f"Error: Cannot delete partition {part.name} on CPC "
                  f"{cpc.name} for clean up - "
                  f"Please delete it manually: {exc.__class__.__name__}: {exc}")
            sys.exit(1)

finally:
    print("Logging off ...")
    session.logoff()



# Copyright 2017,2021 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _user_pattern module.
"""


import re
import copy
import pytest

from zhmcclient import Client, HTTPError, NotFound, UserPattern
from zhmcclient_mock import FakedSession
from tests.common.utils import assert_resources


class TestUserPattern:
    """All tests for the UserPattern and UserPatternManager classes."""

    def setup_method(self):
        """
        Setup that is called by pytest before each test method.

        Set up a faked session, and add a faked Console without any
        child resources.
        """
        # pylint: disable=attribute-defined-outside-init

        self.session = FakedSession('fake-host', 'fake-hmc', '2.13.1', '1.8')
        self.client = Client(self.session)

        self.faked_console = self.session.hmc.consoles.add({
            'object-id': None,
            # object-uri will be automatically set
            'parent': None,
            'class': 'console',
            'name': 'fake-console1',
            'description': 'Console #1',
        })
        self.console = self.client.consoles.find(name=self.faked_console.name)

    def add_user_pattern(self, name, pattern, type_, user_template_uri):
        """
        Add a faked user pattern object to the faked Console and return it.
        """
        faked_user_pattern = self.faked_console.user_patterns.add({
            'element-id': f'oid-{name}',
            # element-uri will be automatically set
            'parent': '/api/console',
            'class': 'user-pattern',
            'name': name,
            'description': f'User Pattern {name}',
            'pattern': pattern,
            'type': type_,
            'retention-time': 0,
            'specific-template-uri': user_template_uri,
        })
        return faked_user_pattern

    def add_user(self, name, type_):
        """
        Add a faked user object to the faked Console and return it.
        """
        faked_user = self.faked_console.users.add({
            'object-id': f'oid-{name}',
            # object-uri will be automatically set
            'parent': '/api/console',
            'class': 'user',
            'name': name,
            'description': f'User {name}',
            'type': type_,
            'authentication-type': 'local',
        })
        return faked_user

    def test_upm_repr(self):
        """Test UserPatternManager.__repr__()."""

        user_pattern_mgr = self.console.user_patterns

        # Execute the code to be tested
        repr_str = repr(user_pattern_mgr)

        repr_str = repr_str.replace('\n', '\\n')
        # We check just the begin of the string:
        assert re.match(
            rf'^{user_pattern_mgr.__class__.__name__}\s+at\s+'
            rf'0x{id(user_pattern_mgr):08x}\s+\(\\n.*',
            repr_str)

    def test_upm_initial_attrs(self):
        """Test initial attributes of UserPatternManager."""

        user_pattern_mgr = self.console.user_patterns

        # Verify all public properties of the manager object
        assert user_pattern_mgr.resource_class == UserPattern
        assert user_pattern_mgr.class_name == 'user-pattern'
        assert user_pattern_mgr.session is self.session
        assert user_pattern_mgr.parent is self.console
        assert user_pattern_mgr.console is self.console

    @pytest.mark.parametrize(
        "full_properties_kwargs, prop_names", [
            (dict(full_properties=False),
             ['element-uri', 'name', 'type']),
            (dict(full_properties=True),
             ['element-uri', 'name', 'type', 'description']),
            ({},  # test default for full_properties (False)
             ['element-uri', 'name', 'type']),
        ]
    )
    @pytest.mark.parametrize(
        "filter_args, exp_names", [
            (None,
             ['a', 'b']),
            ({},
             ['a', 'b']),
            ({'name': 'a'},
             ['a']),
            ({'name': 'A'},  # user patterns have case-insensitive names
             ['a']),
        ]
    )
    def test_upm_list(
            self, filter_args, exp_names, full_properties_kwargs, prop_names):
        """Test UserPatternManager.list()."""

        faked_user1 = self.add_user(name='a', type_='standard')
        faked_user2 = self.add_user(name='b', type_='standard')

        faked_user_pattern1 = self.add_user_pattern(
            name='a', pattern='a_*', type_='glob-like',
            user_template_uri=faked_user1.uri)
        faked_user_pattern2 = self.add_user_pattern(
            name='b', pattern='b_.*', type_='regular-expression',
            user_template_uri=faked_user2.uri)
        faked_user_patterns = [faked_user_pattern1, faked_user_pattern2]
        exp_faked_user_patterns = [u for u in faked_user_patterns
                                   if u.name in exp_names]
        user_pattern_mgr = self.console.user_patterns

        # Execute the code to be tested
        user_patterns = user_pattern_mgr.list(filter_args=filter_args,
                                              **full_properties_kwargs)

        assert_resources(user_patterns, exp_faked_user_patterns, prop_names)

    @pytest.mark.parametrize(
        "input_props, exp_prop_names, exp_exc", [
            ({},  # props missing
             None,
             HTTPError({'http-status': 400, 'reason': 5})),
            ({'description': 'fake description X'},  # props missing
             None,
             HTTPError({'http-status': 400, 'reason': 5})),
            ({'description': 'fake description X',
              'name': 'a',
              'pattern': 'a*'},  # several missing
             None,
             HTTPError({'http-status': 400, 'reason': 5})),
            ({'description': 'fake description X',
              'name': 'a',
              'pattern': 'a*'},  # several missing
             None,
             HTTPError({'http-status': 400, 'reason': 5})),
            ({'description': 'fake description X',
              'name': 'a',
              'pattern': 'a*',
              'type': 'glob-like'},  # props missing
             None,
             HTTPError({'http-status': 400, 'reason': 5})),
            ({'description': 'fake description X',
              'name': 'a',
              'pattern': 'a*',
              'type': 'glob-like',
              'retention-time': 0},  # user template identif. method missing
             None,
             HTTPError({'http-status': 400, 'reason': 15})),
            ({'description': 'fake description X',
              'name': 'a',
              'pattern': 'a*',
              'type': 'glob-like',
              'retention-time': 28,
              'specific-template-uri': '/api/users/oid-tpl'},
             ['element-uri', 'name', 'description', 'pattern', 'type',
              'retention-time', 'specific-template-uri'],
             None),
        ]
    )
    def test_upm_create(self, input_props, exp_prop_names, exp_exc):
        """Test UserPatternManager.create()."""

        faked_user_template = self.add_user(name='tpl', type_='template')
        assert faked_user_template.uri == '/api/users/oid-tpl'

        user_pattern_mgr = self.console.user_patterns

        if exp_exc is not None:

            with pytest.raises(exp_exc.__class__) as exc_info:

                # Execute the code to be tested
                user_pattern_mgr.create(properties=input_props)

            exc = exc_info.value
            if isinstance(exp_exc, HTTPError):
                assert exc.http_status == exp_exc.http_status
                assert exc.reason == exp_exc.reason

        else:

            # Execute the code to be tested.
            user_pattern = user_pattern_mgr.create(properties=input_props)

            # Check the resource for consistency within itself
            assert isinstance(user_pattern, UserPattern)
            user_pattern_name = user_pattern.name
            exp_user_pattern_name = user_pattern.properties['name']
            assert user_pattern_name == exp_user_pattern_name
            user_pattern_uri = user_pattern.uri
            exp_user_pattern_uri = user_pattern.properties['element-uri']
            assert user_pattern_uri == exp_user_pattern_uri

            # Check the properties against the expected names and values
            for prop_name in exp_prop_names:
                assert prop_name in user_pattern.properties
                if prop_name in input_props:
                    value = user_pattern.properties[prop_name]
                    exp_value = input_props[prop_name]
                    assert value == exp_value

    def test_up_repr(self):
        """Test UserPattern.__repr__()."""

        faked_user1 = self.add_user(name='a', type_='standard')
        faked_user_pattern1 = self.add_user_pattern(
            name='a', pattern='a_*', type_='glob-like',
            user_template_uri=faked_user1.uri)
        user_pattern1 = self.console.user_patterns.find(
            name=faked_user_pattern1.name)

        # Execute the code to be tested
        repr_str = repr(user_pattern1)

        repr_str = repr_str.replace('\n', '\\n')
        # We check just the begin of the string:
        assert re.match(
            rf'^{user_pattern1.__class__.__name__}\s+at\s+'
            rf'0x{id(user_pattern1):08x}\s+\(\\n.*',
            repr_str)

    @pytest.mark.parametrize(
        "input_props, exp_exc", [
            ({'name': 'a',
              'description': 'fake description X',
              'pattern': 'a*',
              'type': 'glob-like',
              'retention-time': 28,
              'user-template-uri': '/api/users/oid-tpl'},
             None),
        ]
    )
    def test_up_delete(self, input_props, exp_exc):
        """Test UserPattern.delete()."""

        faked_user_pattern = self.add_user_pattern(
            name=input_props['name'],
            pattern=input_props['pattern'],
            type_=input_props['type'],
            user_template_uri=input_props['user-template-uri'])

        user_pattern_mgr = self.console.user_patterns
        user_pattern = user_pattern_mgr.find(name=faked_user_pattern.name)

        if exp_exc is not None:

            with pytest.raises(exp_exc.__class__) as exc_info:

                # Execute the code to be tested
                user_pattern.delete()

            exc = exc_info.value
            if isinstance(exp_exc, HTTPError):
                assert exc.http_status == exp_exc.http_status
                assert exc.reason == exp_exc.reason

            # Check that the user pattern still exists
            user_pattern_mgr.find(name=faked_user_pattern.name)

        else:

            # Execute the code to be tested.
            user_pattern.delete()

            # Check that the user pattern no longer exists
            with pytest.raises(NotFound) as exc_info:
                user_pattern_mgr.find(name=faked_user_pattern.name)

    def test_up_delete_create_same(self):
        """Test UserPattern.delete() followed by create() with same name."""

        user_pattern_name = 'faked_a'

        faked_user1 = self.add_user(name='a', type_='standard')

        # Add the user pattern to be tested
        self.add_user_pattern(
            name=user_pattern_name, pattern='a_*', type_='glob-like',
            user_template_uri=faked_user1.uri)

        # Input properties for a user pattern with the same name
        sn_user_pattern_props = {
            'name': user_pattern_name,
            'description': 'User Pattern with same name',
            'pattern': 'a*',
            'type': 'glob-like',
            'retention-time': 28,
            'specific-template-uri': '/api/users/oid-tpl',
        }

        user_pattern_mgr = self.console.user_patterns
        user_pattern = user_pattern_mgr.find(name=user_pattern_name)

        # Execute the deletion code to be tested
        user_pattern.delete()

        # Check that the user pattern no longer exists
        with pytest.raises(NotFound):
            user_pattern_mgr.find(name=user_pattern_name)

        # Execute the creation code to be tested.
        user_pattern_mgr.create(sn_user_pattern_props)

        # Check that the user pattern exists again under that name
        sn_user_pattern = user_pattern_mgr.find(name=user_pattern_name)
        description = sn_user_pattern.get_property('description')
        assert description == sn_user_pattern_props['description']

    @pytest.mark.parametrize(
        "input_props", [
            {},
            {'description': 'New user pattern description'},
        ]
    )
    def test_up_update_properties(self, input_props):
        """Test UserPattern.update_properties()."""

        user_pattern_name = 'faked_a'

        faked_user1 = self.add_user(name='a', type_='standard')

        # Add the user pattern to be tested
        self.add_user_pattern(
            name=user_pattern_name, pattern='a_*', type_='glob-like',
            user_template_uri=faked_user1.uri)

        user_pattern_mgr = self.console.user_patterns
        user_pattern = user_pattern_mgr.find(name=user_pattern_name)

        user_pattern.pull_full_properties()
        saved_properties = copy.deepcopy(user_pattern.properties)

        # Execute the code to be tested
        user_pattern.update_properties(properties=input_props)

        # Verify that the resource object already reflects the property
        # updates.
        for prop_name in saved_properties:
            if prop_name in input_props:
                exp_prop_value = input_props[prop_name]
            else:
                exp_prop_value = saved_properties[prop_name]
            assert prop_name in user_pattern.properties
            prop_value = user_pattern.properties[prop_name]
            assert prop_value == exp_prop_value, \
                f"Unexpected value for property {prop_name!r}"

        # Refresh the resource object and verify that the resource object
        # still reflects the property updates.
        user_pattern.pull_full_properties()
        for prop_name in saved_properties:
            if prop_name in input_props:
                exp_prop_value = input_props[prop_name]
            else:
                exp_prop_value = saved_properties[prop_name]
            assert prop_name in user_pattern.properties
            prop_value = user_pattern.properties[prop_name]
            assert prop_value == exp_prop_value



#!/usr/bin/env python
# Copyright 2016,2022 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example that creates a metrics context, retrieves metrics and deletes the
metrics context.
"""

import sys
import time
import requests.packages.urllib3

import zhmcclient
from zhmcclient.testutils import hmc_definitions

requests.packages.urllib3.disable_warnings()

# Get HMC info from HMC inventory and vault files
hmc_def = hmc_definitions()[0]
nickname = hmc_def.nickname
host = hmc_def.host
userid = hmc_def.userid
password = hmc_def.password
verify_cert = hmc_def.verify_cert

print(__doc__)

print(f"Using HMC {nickname} at {host} with userid {userid} ...")

print("Creating a session with the HMC ...")
try:
    session = zhmcclient.Session(
        host, userid, password, verify_cert=verify_cert)
except zhmcclient.Error as exc:
    print(f"Error: Cannot establish session with HMC {host}: "
          f"{exc.__class__.__name__}: {exc}")
    sys.exit(1)

try:
    client = zhmcclient.Client(session)

    try:
        metric_groups = [

            # Please edit this section so it contains only the metric groups
            # that mnatch the operational mode of the targeted CPC.

            'cpc-usage-overview',  # Only in classic mode
            'logical-partition-usage',  # Only in classic mode
            'channel-usage',  # Only in classic mode

            'dpm-system-usage-overview',  # Only in DPM mode
            'partition-usage',  # Only in DPM mode
            'adapter-usage',  # Only in DPM mode
            'crypto-usage',  # Only in DPM mode
            'flash-memory-usage',  # Only in DPM mode
            'roce-usage',  # Only in DPM mode

            # 'environmental-power-status',  # In any mode, starting with z15

            # 'virtualization-host-cpu-memory-usage',  # Only in ensemble mode

        ]

        print("Creating Metrics Context ...")
        mc = client.metrics_contexts.create(
            {'anticipated-frequency-seconds': 15,
             'metric-groups': metric_groups})

        sleep_time = 15  # seconds

        print(f"Sleeping for {sleep_time} seconds ...")
        time.sleep(sleep_time)

        print("Retrieving the current metric values ...")
        mr_str = mc.get_metrics()

        print("Current metric values:")
        mr = zhmcclient.MetricsResponse(mc, mr_str)
        for mg in mr.metric_group_values:
            mg_name = mg.name
            mg_def = mc.metric_group_definitions[mg_name]
            print(f"  Metric group: {mg_name}")
            for ov in mg.object_values:
                print(f"    Resource: {ov.resource_uri}")
                print(f"    Timestamp: {ov.timestamp}")
                print("    Metric values:")
                for m_name in ov.metrics:
                    m_value = ov.metrics[m_name]
                    m_def = mg_def.metric_definitions[m_name]
                    m_unit = m_def.unit or ''
                    print(f"      {m_name:30}  {m_value} {m_unit}")
            if not mg.object_values:
                print("    No resources")

        print("Deleting Metrics Context ...")
        mc.delete()

    except zhmcclient.Error as exc:
        print(f"{exc.__class__.__name__}: {exc}")
        sys.exit(1)

finally:
    print("Logging off ...")
    session.logoff()



#!/usr/bin/env python
# Copyright 2017,2022 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example that shows the OS messages of the OS in a started partition on a CPC
in DPM mode.
"""

import sys
import requests.packages.urllib3
import stomp
import zhmcclient
from zhmcclient.testutils import hmc_definitions

requests.packages.urllib3.disable_warnings()

# Get HMC info from HMC inventory and vault files
hmc_def = hmc_definitions()[0]
nickname = hmc_def.nickname
host = hmc_def.host
userid = hmc_def.userid
password = hmc_def.password
verify_cert = hmc_def.verify_cert

# Print metadata for each OS message, before each message
PRINT_METADATA = False

print(__doc__)

print(f"Using HMC {nickname} at {host} with userid {userid} ...")

print("Creating a session with the HMC ...")
try:
    session = zhmcclient.Session(
        host, userid, password, verify_cert=verify_cert)
except zhmcclient.Error as exc:
    print(f"Error: Cannot establish session with HMC {host}: "
          f"{exc.__class__.__name__}: {exc}")
    sys.exit(1)

try:
    client = zhmcclient.Client(session)

    print("Finding a CPC in DPM mode ...")
    cpcs = client.cpcs.list(filter_args={'dpm-enabled': True})
    if not cpcs:
        print(f"Error: HMC at {host} does not manage any CPCs in DPM mode")
        sys.exit(1)
    cpc = cpcs[0]
    print(f"Using CPC {cpc.name}")

    print(f"Finding an active partition on CPC {cpc.name} ...")
    parts = cpc.partitions.list(filter_args={'status': 'active'})
    if not parts:
        print(f"Error: CPC {cpc.name} does not have any active partitions")
        sys.exit(1)
    part = parts[0]
    print(f"Using partition {part.name} with status "
          f"{part.get_property('status')}")

    print(f"Opening OS message channel for partition {part.name} on CPC "
          f"{cpc.name} (including refresh messages) ...")
    try:
        msg_topic = part.open_os_message_channel(include_refresh_messages=True)
    except zhmcclient.Error as exc:
        print("Error: Cannot open OS message channel for partition "
              f"{part.name}: {exc.__class__.__name__}: {exc}")
        sys.exit(1)
    print(f"OS message channel notification topic: {msg_topic}")

    print(f"Creating a notification receiver for topic {msg_topic} ...")
    try:
        receiver = zhmcclient.NotificationReceiver(
            msg_topic, host, userid, password)
    except Exception as exc:
        print(f"Error: Cannot create notification receiver: {exc}")
        sys.exit(1)

    print(f"Debug: STOMP retry/timeout config: {receiver._rt_config}")

    print("Showing OS messages ...")
    print("-----------------------")
    while True:
        try:
            for headers, message in receiver.notifications():
                os_msg_list = message['os-messages']
                for os_msg in os_msg_list:
                    if PRINT_METADATA:
                        msg_id = os_msg['message-id']
                        held = os_msg['is-held']
                        priority = os_msg['is-priority']
                        prompt = os_msg.get('prompt-text', None)
                        print(f"# OS message {msg_id} (held: {held}, "
                              f"priority: {priority}, prompt: {prompt}):")
                    msg_txt = os_msg['message-text'].strip('\n')
                    print(msg_txt)
        except zhmcclient.NotificationError as exc:
            print(f"Notification Error: {exc} - reconnecting")
            continue
        except stomp.exception.StompException as exc:
            print(f"STOMP Error: {exc} - reconnecting")
            continue
        except KeyboardInterrupt:
            print("Keyboard interrupt - leaving receiver loop")
            receiver.close()
            break
        else:
            raise AssertionError("Receiver was closed - should not happen")

finally:
    print("Logging off ...")
    session.logoff()



# Copyright 2017,2021 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=attribute-defined-outside-init

"""
Unit test cases for the tests/common/utils.py module.
"""


import pytest

from zhmcclient import Client
from zhmcclient_mock import FakedSession

from tests.common.utils import assert_resources


class TestUtilsAssertResources:
    """All tests for utils.assert_resources()."""

    def setup_method(self):
        """
        Called by pytest before each test method.

        Creates a Faked session (including HMC) and client.
        """
        self.session = FakedSession('fake-host', 'fake-hmc', '2.13.1', '1.8')
        self.client = Client(self.session)

    def add_cpcs(self):
        """Add faked CPCs to the faked HMC."""

        faked_cpc1 = self.session.hmc.cpcs.add({
            'object-id': 'fake-cpc1-oid',
            # object-uri is auto-generated
            'parent': None,
            'class': 'cpc',
            'name': 'fake-cpc1-name',
            'description': 'CPC #1',
            'status': 'active',
            'dpm-enabled': True,
            'is-ensemble-member': False,
            'iml-mode': 'dpm',
        })
        faked_cpc2 = self.session.hmc.cpcs.add({
            'object-id': 'fake-cpc2-oid',
            # object-uri is auto-generated
            'parent': None,
            'class': 'cpc',
            'name': 'fake-cpc2-name',
            'description': 'CPC #2',
            'status': 'active',
            'dpm-enabled': False,
            'is-ensemble-member': False,
            'iml-mode': 'lpar',
        })
        return [faked_cpc1, faked_cpc2]

    @pytest.mark.parametrize(
        "reverse", [False, True]
    )
    @pytest.mark.parametrize(
        "props", ['all', 'some', 'empty', 'none']
    )
    def test_assert_resources_success(self, props, reverse):
        """Test assert_resources() with successful parameters."""

        faked_cpcs = self.add_cpcs()
        cpcs = self.client.cpcs.list(full_properties=True)

        resources = cpcs
        exp_resources = faked_cpcs
        if reverse:
            exp_resources = list(reversed(exp_resources))

        if props == 'all':
            prop_names = exp_resources[0].properties.keys()
        elif props == 'some':
            prop_names = ['name', 'status']
            # We change a property that is not being checked:
            exp_resources[0].properties['description'] = 'changed description'
        elif props == 'empty':
            prop_names = []
            # No properties are checked, we change a property:
            exp_resources[0].properties['description'] = 'changed description'
        else:
            assert props == 'none'
            # All properties are checked.
            prop_names = None

        # Execute the code to be tested
        assert_resources(resources, exp_resources, prop_names)

        # If it comes back, our test is successful.

    @pytest.mark.parametrize(
        "reverse", [False, True]
    )
    @pytest.mark.parametrize(
        "props", ['all', 'some', 'none']
    )
    def test_assert_resources_error_props(self, props, reverse):
        """Test assert_resources() with failing property checks."""

        faked_cpcs = self.add_cpcs()
        cpcs = self.client.cpcs.list(full_properties=True)

        resources = cpcs
        exp_resources = faked_cpcs
        if reverse:
            exp_resources = list(reversed(exp_resources))

        if props == 'all':
            prop_names = exp_resources[0].properties.keys()
            # We change a property that is being checked:
            exp_resources[0].properties['description'] = 'changed description'
        elif props == 'some':
            prop_names = ['name', 'status']
            # We change a property that is being checked:
            exp_resources[0].properties['status'] = 'not-operating'
        else:
            assert props == 'none'
            # All properties are checked.
            prop_names = None
            # We change a property that is being checked:
            exp_resources[0].properties['description'] = 'changed description'

        # Execute the code to be tested
        with pytest.raises(AssertionError):
            assert_resources(resources, exp_resources, prop_names)

    @pytest.mark.parametrize(
        "reverse", [False, True]
    )
    def test_assert_resources_error_res(self, reverse):
        """Test assert_resources() with failing resource list checks."""

        faked_cpcs = self.add_cpcs()
        cpcs = self.client.cpcs.list(full_properties=True)

        resources = cpcs[0:1]   # trigger non-matching resource list
        exp_resources = faked_cpcs
        if reverse:
            exp_resources = list(reversed(exp_resources))

        prop_names = exp_resources[0].properties.keys()

        # Execute the code to be tested
        with pytest.raises(AssertionError):
            assert_resources(resources, exp_resources, prop_names)



# Copyright 2016,2021 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _virtual_switch module.
"""


import re
# FIXME: Migrate requests_mock to zhmcclient_mock.
import requests_mock

from zhmcclient import Session, Client, Nic


class TestVirtualSwitch:
    """All tests for VirtualSwitch and VirtualSwitchManager classes."""

    def setup_method(self):
        """
        Setup that is called by pytest before each test method.
        """
        # pylint: disable=attribute-defined-outside-init

        self.session = Session('vswitch-dpm-host', 'vswitch-user',
                               'vswitch-pwd')
        self.client = Client(self.session)
        with requests_mock.mock() as m:
            # Because logon is deferred until needed, we perform it
            # explicitly in order to keep mocking in the actual test simple.
            m.post(
                '/api/sessions', json={
                    'api-session': 'test-session-id',
                    'notification-topic': 'test-obj-topic.1',
                    'job-notification-topic': 'test-job-topic.1',
                    'session-credential':
                        'un8bu462g37aw9j0o8pltontz3szt35jh4b1qe2toxt6fkhl4',
                })
            m.get(
                '/api/version', json={
                    'api-major-version': 4,
                    'api-minor-version': 10,
                })
            self.session.logon()

        self.cpc_mgr = self.client.cpcs
        with requests_mock.mock() as m:
            result = {
                'cpcs': [
                    {
                        'object-uri': '/api/cpcs/vswitch-cpc-id-1',
                        'name': 'CPC',
                        'status': 'service-required',
                    }
                ]
            }
            m.get('/api/cpcs', json=result)

            cpcs = self.cpc_mgr.list()
            self.cpc = cpcs[0]

    def teardown_method(self):
        """
        Teardown that is called by pytest after each test method.
        """
        with requests_mock.mock() as m:
            m.delete('/api/sessions/this-session', status_code=204)
            self.session.logoff()

    def test_init(self):
        """Test __init__() on VirtualSwitchManager instance in CPC."""
        vswitch_mgr = self.cpc.virtual_switches
        assert vswitch_mgr.cpc == self.cpc

    def test_list_short_ok(self):
        """
        Test successful list() with short set of properties
        on VirtualSwitchManager instance in CPC.
        """
        vswitch_mgr = self.cpc.virtual_switches
        with requests_mock.mock() as m:
            result = {
                'virtual-switches': [
                    {
                        'name': 'VSWITCH1',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id1',
                        'type': 'osd'
                    },
                    {
                        'name': 'VSWITCH2',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id2',
                        'type': 'hpersockets'
                    }
                ]
            }

            m.get('/api/cpcs/vswitch-cpc-id-1/virtual-switches', json=result)

            vswitches = vswitch_mgr.list(full_properties=False)

            assert len(vswitches) == len(result['virtual-switches'])
            for idx, vswitch in enumerate(vswitches):
                assert vswitch.properties == \
                    result['virtual-switches'][idx]
                assert vswitch.uri == \
                    result['virtual-switches'][idx]['object-uri']
                assert not vswitch.full_properties
                assert vswitch.manager == vswitch_mgr

    def test_list_full_ok(self):
        """
        Test successful list() with full set of properties on
        VirtualSwitchManager instance in CPC.
        """
        vswitch_mgr = self.cpc.virtual_switches
        with requests_mock.mock() as m:
            result = {
                'virtual-switches': [
                    {
                        'name': 'VSWITCH1',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id1',
                        'type': 'osd'
                    },
                    {
                        'name': 'VSWITCH2',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id2',
                        'type': 'hipersockets'
                    }
                ]
            }

            m.get('/api/cpcs/vswitch-cpc-id-1/virtual-switches', json=result)

            mock_result_get_bulk = [
                {
                    'id': '1',
                    'status': 200,
                    'body': {
                        'name': 'VSWITCH1',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id1',
                        'type': 'osd',
                        'class': 'virtual-switch',
                        'description': 'Test VirtualSwitch',
                        'more_properties': 'bliblablub'
                    }
                },
                {
                    'id': '2',
                    'status': 200,
                    'body': {
                        'name': 'VSWITCH2',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id2',
                        'type': 'hipersockets',
                        'class': 'virtual-switch',
                        'description': 'Test VirtualSwitch',
                        'more_properties': 'bliblablub'
                    }
                }
            ]
            m.post('/api/services/aggregation/submit',
                   json=mock_result_get_bulk)

            vswitches = vswitch_mgr.list(full_properties=True)

            assert len(vswitches) == len(result['virtual-switches'])
            for idx, vswitch in enumerate(vswitches):
                assert vswitch.properties['name'] == \
                    result['virtual-switches'][idx]['name']
                assert vswitch.uri == \
                    result['virtual-switches'][idx]['object-uri']
                assert vswitch.full_properties
                assert vswitch.manager == vswitch_mgr

    def test_list_add_props(self):
        """
        Test successful list() with additional_properties.
        """
        vswitch_mgr = self.cpc.virtual_switches
        with requests_mock.mock() as m:
            result = {
                'virtual-switches': [
                    {
                        'name': 'VSWITCH1',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id1',
                        'type': 'osd',
                        'port': 1,
                    },
                    {
                        'name': 'VSWITCH2',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id2',
                        'type': 'hipersockets',
                        'port': 0,
                    }
                ]
            }

            m.get('/api/cpcs/vswitch-cpc-id-1/virtual-switches', json=result)

            vswitches = vswitch_mgr.list(additional_properties=['port'])

            assert len(vswitches) == len(result['virtual-switches'])
            for idx, vswitch in enumerate(vswitches):
                assert vswitch.properties['name'] == \
                    result['virtual-switches'][idx]['name']
                assert vswitch.uri == \
                    result['virtual-switches'][idx]['object-uri']
                assert 'port' in vswitch.properties
                assert vswitch.properties['port'] == \
                    result['virtual-switches'][idx]['port']
                assert vswitch.manager == vswitch_mgr

    def test_update_properties(self):
        """
        This tests the 'Update VirtualSwitch Properties' operation.
        """
        vswitch_mgr = self.cpc.virtual_switches
        with requests_mock.mock() as m:
            result = {
                'virtual-switches': [
                    {
                        'name': 'VSWITCH1',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id1',
                        'type': 'osd'
                    },
                    {
                        'name': 'VSWITCH2',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id2',
                        'type': 'hpersockets'
                    }
                ]
            }
            m.get('/api/cpcs/vswitch-cpc-id-1/virtual-switches', json=result)

            vswitches = vswitch_mgr.list(full_properties=False)
            vswitch = vswitches[0]
            m.post("/api/virtual-switches/fake-vswitch-id1", status_code=204)
            vswitch.update_properties(properties={})

    def test_get_connected_nics(self):
        """
        This tests the `get_connected_nics()` method.
        """
        vswitch_mgr = self.cpc.virtual_switches
        with requests_mock.mock() as m:
            result = {
                'virtual-switches': [
                    {
                        'name': 'VSWITCH1',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id1',
                        'type': 'osd'
                    },
                    {
                        'name': 'VSWITCH2',
                        'object-uri': '/api/virtual-switches/fake-vswitch-id2',
                        'type': 'hpersockets'
                    }
                ]
            }
            m.get('/api/cpcs/vswitch-cpc-id-1/virtual-switches', json=result)

            vswitches = vswitch_mgr.list(full_properties=False)
            vswitch = vswitches[0]
            result = {
                'connected-vnic-uris': [
                    '/api/partitions/fake-part-id1/nics/fake-nic-id1',
                    '/api/partitions/fake-part-id1/nics/fake-nic-id2',
                    '/api/partitions/fake-part-id1/nics/fake-nic-id3'
                ]
            }

            m.get(
                "/api/virtual-switches/fake-vswitch-id1/"
                "operations/get-connected-vnics",
                json=result)

            nics = vswitch.get_connected_nics()

            assert isinstance(nics, list)
            for i, nic in enumerate(nics):
                assert isinstance(nic, Nic)
                nic_uri = result['connected-vnic-uris'][i]
                assert nic.uri == nic_uri
                assert nic.properties['element-uri'] == nic_uri
                m = re.match(r"^/api/partitions/([^/]+)/nics/([^/]+)/?$",
                             nic_uri)
                nic_id = m.group(2)
                assert nic.properties['element-id'] == nic_id
